%f_step_on_rake_revision.m
% function for stepping on rake calculations
% step_on_rake_JC_revision sets up parameters and uses this function to make graphs
%
% A typical call: 
% sims parameters: 
%    gam = 0.5;
%    thet = 0.4;
%    phi = 0.75;
%    sig = 2.0;
%    rhobar = 0.05;
%    rhohat = 0.05; % set to zero to replicate sims typo
%    taubar = 0.10;
%    bet = 0.10;
%    delt = 0.20;
%    omeg = 1; 
%    psi = 2;
%    longdebt = 1; 
%
% em = 0.001; % monetary policy shock. B is not linear, so smaller is better. Other variables don't care
% etau = -0.001; % fiscal shock. 
% T = 15; 
% dt = 0.01; 

% Make these shocks small. Program reports
%result/em. B calculation is nonlinear and wants small shocks; others don't care 
 

function [eigs,t,rt, pit,at, taut,bt,lamt,ct,cdott,pt,rhot,Bt,...
    ta,rta, pita,ata, tauta,bta,lamta,cta,cdotta,pta,rhota,Bta] =...
    f_step_on_rake_revision(gam,thet,phi,sig,rhobar,rhohat,taubar,bet,delt,omeg,psi,...
    longdebt,T,dt,em,etau)

b = taubar/rhobar;

if (psi > 0 )& (max(size(phi))==1); % allows program to run with old calls that only have x dot dependence
    phi = [phi; 0]; 
end


% transition matrix dx / dt = A x + eps

if psi > 0
    %       r     tau   pi   y  b      lam     cdot    c
    A = [-gam    0     thet  0      0       0      phi(1)  phi(2)   ; % r
        0        0     0     0      0       0      omeg    0   ; %tau
        0        0     bet   0      0       0      0      -delt   ; %pi
        -rhobar  0      0    rhobar 0       0      0       0       ; % y
        b       -1     -b    0      rhobar  0      0       0      ; %b
        -1       0      1    0      0       0      0       0       ; %lam
        0        0      0    0      0       1/psi  rhobar sig/psi ; %cdot
        0        0      0    0      0        0     1       0];      %c
end;
if psi == 0;   
    
    %      r         tau   pi           y   b        c  
    A = [phi/sig-gam  0  thet-phi/sig   0       0        0     ; % r
         omeg/sig     0  -omeg/sig      0       0        0     ; %tau
         0            0  bet            0       0       -delt ; %pi
         -rhobar      0   0             rhobar  0        0     ; %y  
         b           -1  -b             0       rhobar   0     ;%b
        1/sig         0  -1/sig         0       0        0     ]; %c
end;
 
[Q, Lam] = eig(A);
eigs = diag(Lam);
posindx = find(real(diag(Lam))>0); %index of positive eigenvalues
if 1;
    disp('Real | imaginary part of Eigenvalues');
    disp(real(diag(Lam)'));
    disp(imag(diag(Lam)'));
    disp('number of unstable eigenvalues');
    disp(size(posindx,1));
end
nonposindx = find(real(diag(Lam))<=0);
lambdas = diag(Lam);
lambdas_neg = lambdas(nonposindx); % stable eigenvalues
invQ = inv(Q);
Qmat = invQ(posindx,:);

 
if psi > 0;
    selmat = [ 0 0 0 0 ; 
        0 0 0 0 ; 
        1 0 0 0 ; 
        0 1 0 0 ; 
        0 -longdebt*b/rhobar 0 0 ; 
        0 0 1 0 ; 
        0 0 0 1 ; 
        0 0 0 0 ; ];  
     shockv = [ em ; etau ; 0; 0; 0; 0; 0; 0 ]; 
     deltas = - inv(Qmat*selmat)*Qmat*shockv; % the pi, y lam, c
     epsilon = selmat*deltas + shockv; % this is the full shock vector
end;

if (psi == 0) & (bet > 0);
    selmat = [ 0 0 phi  ;
         0 0 omeg ;
         1 0 0  ;
         0 1 0  ;
         0 -longdebt*b/rhobar 0; 
         0 0 1 ]; 
     shockv = [ em; etau ; 0; 0; 0; 0 ]; 
     deltas = - inv(Qmat*selmat)*Qmat*shockv; 
     epsilon = selmat * deltas + shockv; % this is the full shock vector
end;

if (psi == 0) & (bet < 0);
    selmat = [ 0 phi  ;
         0 omeg ;
         0 0  ;
         1 0  ;
         -longdebt*b/rhobar 0; 
         0 1 ]; 
     shockv = [ em; etau ; 0; 0; 0; 0 ]; 
     deltas = - inv(Qmat*selmat)*Qmat*shockv; 
     epsilon = selmat * deltas + shockv; % this is the full shock vector
end;


 x0 = epsilon;
 %y0_check = invQ*x0;
 Qmatneg = invQ(nonposindx,:);
 y0 = Qmatneg*x0; 
 
 Qneg = Q(:,nonposindx); 
 
t = (0:dt:T)';
yt = exp(t*lambdas_neg').*(ones(size(t,1),1)*y0');
xt = yt*Qneg';

if psi > 0;
    rt = xt(:,1);
    taut = xt(:,2);
    pit = xt(:,3);
    at = xt(:,4); 
    bt = xt(:,5);
    lamt = xt(:,6);
    cdott = xt(:,7);
    ct = xt(:,8);
end;
if psi == 0;
    rt = xt(:,1);
    taut = xt(:,2);
    pit = xt(:,3);
    at = xt(:,4); 
    bt = xt(:,5);
    ct = xt(:,6);
    lamt = -sig*ct; 
    cdott = (ct(2:end)-ct(1:end-1))/dt;
end;

pt = cumsum([0;pit(2:end)])*dt;

rhot = rt - pit; 


% you can only do linear transformations of differences
% from steady state variables. To find response of
% face value debt, must do it on the levels. 
% use P=1 p=0 as steady state. 

alevt = at + rhobar;
blevt = bt + b;
if longdebt == 1;
    Blevt = blevt.*alevt.*exp(pt); % b = B/aP so B = b*a*P;
    Bt = Blevt - b*rhobar;
elseif longdebt == 0;
    Blevt = blevt.*exp(pt); % b = B/P
    Bt = Blevt - b;
end
 
alevt = at + rhobar;
blevt = bt + b;
if longdebt == 1;
    Blevt = blevt.*alevt.*exp(pt); % b = B/aP so B = b*a*P;
    Bt = Blevt - b*rhobar;
elseif longdebt == 0;
    Blevt = blevt.*exp(pt); % b = B/P
    Bt = Blevt - b;
end
 
% units of b are gdp -- b = 2 means real market value of debt is 2 x GDP. 
% units of B number of perpetuities each of which pays 1 year GDP as a
% coupon. Thus B = 2*a*1 = 2 * 0.05 * 1 = 0.1 means at P=1, rho = 0.05, 
% coupons are 0.1% of GDP 

 % ******************
 % Expected monetary policy 
 % *****************
 
posindx = find(diag(Lam)>0); %index of positive eigenvalues
negindx = find(diag(Lam)<0);
zeroindx = find(diag(Lam)==0); 
lambdas = diag(Lam);
invQ = inv(Q);
       
ta = (-T:dt:T)';
t0ind = find(ta==0); 

yt = zeros(size(ta,1),size(Q,1)); 
lambdas_pos = lambdas(lambdas>0);
lambdas_neg = lambdas(lambdas<0);

yt(1:t0ind,posindx) = exp(ta(1:t0ind)*(lambdas_pos')) .*...
             (-ones(t0ind,1)*(invQ(posindx,:)*shockv)');
yt(t0ind+1:end,negindx) = exp(ta(t0ind+1:end)*(lambdas_neg')) .*...
             (ones(t0ind-1,1)*(invQ(negindx,:)*shockv)');

% for step function i, want second choice -- makes i go from 0 to 1 not -1
% to 0 
% for gamma  neq 0, it doesn't matter which you do. 

%yt(1:t0ind,zeroindx) = (-ones(t0ind,1)*(invQ(zeroindx,:)*shockv)');
yt(t0ind+1:end,zeroindx) = (ones(t0ind-1,1)*(invQ(zeroindx,:)*shockv)');

xt = yt*Q';


if psi > 0;
    rta = xt(:,1);
    tauta = xt(:,2);
    pita = xt(:,3);
    ata = xt(:,4);
    bta = xt(:,5);
    lamta = xt(:,6);
    cdotta = xt(:,7);
    cta = xt(:,8);
    cta = cumsum([0;cdotta(2:end)])*dt;
end;
if psi == 0;
    rta = xt(:,1);
    tauta = xt(:,2);
    pita = xt(:,3);
    ata = xt(:,4);
    bta = xt(:,5);
    cta = xt(:,6);
    lamta = -sig*ct;
    cdotta=[0; (ct(2:end)-ct(1:end-1))/dt];
end;

pta = cumsum([0;pita(2:end)])*dt;
rhota = rta - pita; 

% you can only do linear transformations of differences
% from steady state variables. To find response of
% face value debt, must do it on the levels. 
% use P=1 p=0 as steady state. 

alevt = ata + rhobar;
blevt = bta + b;
if longdebt == 1;
    Blevt = blevt.*alevt.*exp(pta); % b = B/aP so B = b*a*P;
    Bta = Blevt - b*rhobar;
elseif longdebt == 0;
    Blevt = blevt.*exp(pta); % b = B/P
    Bta = Blevt - b;
end
 

% units of b are gdp -- b = 2 means real market value of debt is 2 x GDP. 
% units of B number of perpetuities each of which pays 1 year GDP as a
% coupon. Thus B = 2*a*1 = 2 * 0.05 * 1 = 0.1 means at P=1, rho = 0.05, 
% coupons are 0.1% of GDP 

