% final_plots. Creates selected plots for fiscal theory of the price level, in 
% grayscale, in order. 

% Makes

% Figure 2.1 p. 37
% Figure 2.2 p. 41 
% Figure 3.1 p. 70 
% Figure 7.1 p. 213
% Figure 7.5 p. 229 
% Figure 4.1 p. 82 
% Figure 4.2 p. 84 
% Figure 5.5. p. 197
% Figure 16.1 p. 359
% Figure 17.1 p. 393 
% Figure 17.2 p. 394 
% Figure 17.3 p. 398 
% Figure 17.4 p. 400
% Figure 17.5 p. 402 
% Figure 17.6 p. 403 
% Figure 17.7 p. 406 
% Figure 17.8 p. 409 
% Figure 19.1 p. 436
% Figure 20.1 p. 459
% Figire 20.2 p. 459
% Figure 20.3 p. 465
% Figure 21.1 p. 479 
% Figure 21.2 p. 480
% Figure 5.4 p. 127 
% Figure 5.2 p. 123
% Figure 5.1 p. 121 
% Figure 5.3 p. 124
% Figure 7.6 p. 233

% Figure x p. y refers to final book, not manuscript. Book figures are a
% bit different thanks to PUP visual cleanup. 

clear all
close all

font_size = 16;
black = [ 0 0 0] ; 
gray1 = [ 0 0 0 ] + 0.5; 
allplots = 1; % used to work on a selected plot.  
global global_pars 

%%%%%%%%%%%%
% From nk_vs_ftpl_simple

% Figure 2.1 p. 37

if allplots;
    T = 50;
    
    % condition on interest rate.
    
    pit = zeros(T,1);
    it = zeros(T,1);
    t1 = 5; % date of shock, plotted as t=1
    ta = 2; % date of announcement
    t0 = -3; % first date to plot
    it(t1:end) = 1;
    pit(t1+1:end) = 1;
    
    pitf = zeros(T,1);
    pitf(t1) = -1;
    
    pitfe = zeros(T,1);
    pitfe(ta) = -1*0.95^(t1-ta);
    
    
    figure
    subplot(2,1,1)
    
    hold on
    plot((t0:t0+T-1)',pit,'-','color',black,'linewidth',2);
    plot((t0:t0+T-1)',it+0.02,'--o','color',gray1,'linewidth',2);
    axis([t0 5 -0.1 1.1])
    text(0.4,0.6,'i_t','color', gray1,'horizontalalignment','r','fontsize',font_size);
    text(1.7,0.5,'\pi_t','color',black ,'fontsize',font_size)
    
    ylabel('Percent')
    
    subplot(2,1,2);
    hold on
    
    plot((t0:t0+T-1)',-pitf,'-','color',black,'linewidth',2);
    plot((t0:t0+T-1)',-pitfe+0.02,'--v','color',gray1,'linewidth',2);
    
    axis([t0 5 -0.1 1.1])
    xlabel('Time')
    ylabel('Percent')
    text(1.3,+0.8,'\pi_t, fiscal shock','color',black,'fontsize',font_size)
    text(-2.7,+.97,'\pi_t, expected fiscal shock', 'color',gray1,'fontsize',font_size)
    
    print -depsc2 simple_i_response.eps
    
end;

%%%%%%%%%%%%%%%%%%%%

% PLOT FTMP with Taylor rules 
% fiscal theory, moentary shock

% Figure 2.2 p. 41 

if allplots;
    phi = 0.8;
    rho = 0.7;
    
    pit = zeros(T,1);
    vt = zeros(T,1);
    it = zeros(T,1);
    est = zeros(T,1);
    eit = zeros(T,1);
    eit(2) = 1;
    est(2) = 0;
    
    for t = 1:T-1;
        pit(t+1) = phi*pit(t) + vt(t) - est(t+1);
        vt(t+1) = rho*vt(t) + eit(t+1);
        it(t) = phi*pit(t) + vt(t);
    end;
    it(T) = phi*pit(T)+vt(T);
    
    figure
    subplot(2,1,1);
    
    hold on;
    plot((0:T-1)',0*pit,'-k')
    plot((0:T-1)',pit,'-','color',black,'linewidth',2);
    plot((0:T-1)',vt,'-k','linewidth',1.5);
    plot((0:T-1)',it,'--o','color',gray1,'linewidth',2);
    %legend('\pi','v','i');
    text(11,1,'\pi_t','color','k','fontsize',font_size)
    text(8.9,0.8,'i_t','color',gray1,'fontsize',font_size,'horizontalalignment','r')
    text(3,0.7,'u_t','color','k','fontsize',font_size)
    
    axis([0 16 -0.1 1.8 ])
    ylabel('Percent')
    %title('Response to a persistent interest rate shock')
    
    subplot(2,1,2);
    % fiscal theory, fiscal shock
    hold on;
    pitf = zeros(T,1);
    vtf = zeros(T,1);
    itf = zeros(T,1);
    estf = zeros(T,1);
    eitf = zeros(T,1);
    eitf(2) = 0;
    estf(2) = 1;
    
    for t = 1:T-1;
        pitf(t+1) = phi*pitf(t) + vtf(t) - estf(t+1);
        vtf(t+1) = rho*vtf(t) + eitf(t+1);
        itf(t) = phi*pitf(t) + vtf(t);
    end;
    itf(T) = phi*pitf(T)+vtf(T);
    
    
    %figure
    plot((0:T-1)',-pitf,'-k','linewidth',2);
    hold on;
    
    %plot((0:T-1)',vtf,'-m','linewidth',2);
    plot((0:T-1)',-itf,'--o','color',gray1,'linewidth',2);
    %legend('\pi','v','i');
    axis([0 16 -0.1 1.1 ])
    xlabel('Time')
    ylabel('Percent')
    text(4, 0.6,'\pi_t','color','k','fontsize',font_size)
    text(3.5,0.3,'i_t','color',gray1,'fontsize',font_size,'horizontalalignment','r')
    %title('Response to a fiscal shock')
    
    print -depsc2 nk_simple_with_rule.eps
end;

%%%%%%%%%%%%%%%%%%%

% from frictionless2.m

% Figure 3.1 p. 70 

if allplots ;
    omeg = 0.8;
    Ntim = 1000;
    tim = (-10:Ntim)';
    t0 = find(tim==0); % index of time 0
    ntim = size(tim,1);
    
    % Version that uses linearized identity l suffix for linearized
    
    itl = zeros(ntim,1);
    itl(t0+1:end)=1; % change to shock at time 1
    pitl = itl;
    pitl(t0+1) = -omeg/(1-omeg);
    ptl = cumsum(pitl);
    pitnol = itl;
    pitnol(t0+1) = 0;
    ptnol = cumsum(pitnol);
    itl = itl+2;
    
    figure
    
    plot(tim,itl,'--k','linewidth',2);
    hold on
    plot(tim, ptl ,'-k','linewidth',2);
    plot(tim, ptnol,'-o','color',gray1,'linewidth',2);
    
    plot(tim,0*tim,'-k');
    text(-0.3,2.5,'i_t','fontsize',font_size,'color','k');
    text(2.1,-2,'p_t','fontsize',font_size,'color','k');
    text(1.9,1.2,'p_t, short debt or expected',...
        'fontsize',font_size,'color',gray1,'horizontalalignment','r')
    xlabel('Time');
    ylabel('Percent');
    plot([1 1]',[-4 4],'--k');
    axis([-3 8 -4 4]);
    print -depsc2 lt_debt_example.eps
    
end;

% Figure 7.1 p. 213

if allplots; % from frictionless, not frictionless2
    
    r = 0.02; % steady state, pre zero value
    i = 0.03;
    theta = 0.8; % omega in new notation
    s = 1;
    P = 1;
    
    tim = (-10:1000)';
    t0 = find(tim==0); % index of time 0
    ntim = size(tim,1);
    
    it = zeros(ntim,1);
    it(1:t0-1)=r;
    it(t0:end)=i;
        
    T = 3; % announced at time T
    P0T = 1+ (theta/(1+r))^T*((1+i)/(1+r) * (1+r-theta)/(1+i-theta)-1);
    PTt = zeros(ntim,1);
    PTt(1:t0-T-1) = P;
    PTt(t0-T:t0-1)=P0T;
    PTt(t0:end)=P0T*((1+i)/(1+r)).^tim(t0:end);
    
    qtT = [ cumsum(-log(1+it(t0-T-1:end-1)))]; % bond price on day 0
    % qt is the log bond price at time 1 of bonds coming due at time 1:end
    ytT = [-qtT./(1:size(qtT,1))'];
    YtT = exp(ytT)-1;
    
    figure
    plot(tim+T,100*it,'--k','linewidth',2);
    hold on
    plot(tim+T,100*log(PTt),'-ks','linewidth',2);
    plot(tim(t0-T:end)+T,100*ytT,'--^','color',gray1,'linewidth',2)
    
    plot(tim+T,0*tim,'-k');
    plot([0 0]',[-4 4],'--k');
    text(1.7,2.5,'i','fontsize',font_size,'color','k');
    text(3.8,-1.3,'p','fontsize',font_size,'color','k');
    text(3,1.7,'Yields at t=0','color',gray1,'fontsize',font_size);
    xlabel('Time, maturity');
    ylabel('Percent');
    
    axis([-2 10 -2.5 3.5]);
    
    print -depsc2 geom_example_2.eps
end;
    
% Figure 7.5 p. 229 

if allplots; % from frictionless2 again, forward guidance/QE
    
    r = 0.02; % steady state, pre zero value
    i = 0.01; % use i = 0.03 for positive i = 0.01 for negative
    % for now, first two graphs are positie, debt graph is negative.
    % must be set manually.
    theta = 0.8;  % changed to omega in paper
    omeg = theta;
    T = 3; % i rises at time T, P starts rising at time T+1
    
    P = 1;
    R = 1+r;
    Pi = (1+i)/(1+r);
    
    Ntim = 1000;
    tim = (-10:Ntim)';
    t0 = find(tim==0); % index of time 0
    ntim = size(tim,1);
    
    % bonds b tilde / B
    %     [   1   2   3   4   5   6   7 8   9   10 ] due date
    Btil = zeros(Ntim,1);
    Btil(1:10) =-0.05* [ 0.0 0.0 0.0 1.0 1.0 1.0 1.0 0.0 0.0 0.0 ]'; % manual
    %Btil = 0.1^theta.^(0:Ntim-1)'; % geometric
    
    it = zeros(ntim,1);
    it(1:t0-1)=r;
    it(t0:end)=i;

    
    %P0T = 1+ (theta/(1+r))^T*((1+i)/(1+r) * (1+r-theta)/(1+i-theta)-1);
    % from frictionless. They are the same
    itT = zeros(ntim,1);
    itT(1:t0+T-1)=r;
    itT(t0+T:end)=i;
    P0T = 1-(Pi-1)/(R*Pi-theta)*theta*(theta/R)^T;
    PTt = zeros(ntim,1);
    PTt(1:t0-1) = P;
    PTt(t0:t0+T)=P0T;
    PTt(t0+T:end)=P0T*(Pi).^(tim(t0+T:end)-T);
    % these are rewritten from frictionless to be forward guideance
    % not expected, announced at 0 and take place at T
    qtT = [ cumsum(-log(1+itT(t0:end)))]; % bond price on day 0
    % qt is the log bond price at time 0 of bonds coming due at time 1:end
    ytT = [-qtT./(1:size(qtT,1))'];
    YtT = exp(ytT)-1;
    
    % Debt calculations
    % delay case    
    
    Mt1 = zeros(Ntim,1); % this is M t-1
    Mt1_noB = zeros(Ntim,1); % this is M t-1   with no B
    Btilval = zeros(Ntim,1); % market value of the b tildes
    thetaval = zeros(Ntim,1); % market value of the original debt
    thetaval_0 = R/(R-theta)*(1+(1-Pi)/(R*Pi-theta)*(theta/R)^(T+1));
    for t = 1:T;
        Mt1(t) = theta*R*(Pi-1)/((R-theta)*(R*Pi-theta))*((theta/R)^(T-t)-(theta/R)^T)-...
            sum(1./R.^(0:T-t)'.*Btil(t:T))-1/Pi^(t-T)*sum(1./(R*Pi).^(T+1-t:Ntim-t)'.*Btil(T+1:Ntim));
        Mt1_noB(t) = theta*R*(Pi-1)/((R-theta)*(R*Pi-theta))*((theta/R)^(T-t)-(theta/R)^T);
        Btilval(t) = P0T*sum(1./R.^(0:Ntim-t)'.*Btil(t:Ntim)./PTt(t0+t:end));
        thetaval(t) = R/(R-theta)*(1+(1-Pi)/(R*Pi-theta)*(theta/R)^(T-t+1));
    end;
       
    
    for t = T+1:Ntim;
        Mt1(t) = (1-(Pi-1)/(R*Pi-theta)*theta*(theta/R)^T)*R/(R-theta)*Pi^(t-T) - R*Pi/(R*Pi-theta) - ...
            sum(1./(R*Pi).^(0:Ntim-t)'.*Btil(t:Ntim));
        Mt1_noB(t) = (1-(Pi-1)/(R*Pi-theta)*theta*(theta/R)^T)*R/(R-theta)*Pi^(t-T) - R*Pi/(R*Pi-theta);
        Btilval(t) = P0T*sum(1./R.^(0:Ntim-t)'.*Btil(t:Ntim)./PTt(t0+t:end));
        thetaval(t) = R*Pi/(R*Pi-theta);
    end;
    
    % Figure 7.5 p. 229 
    
    figure;
    box on
    hold on
    plot(tim,0*tim,'-k');
    plot([0 0]',[-8 8],'--k');
    plot((1:Ntim)',100*(1-theta/R)*Mt1_noB,'-','color','k','linewidth',2);
    plot((1:Ntim)',100*(1-theta/R)*Btil,'-','color','k','linewidth',2);
    plot((1:Ntim)',100*(1-theta/R)*Mt1,'-s','color',gray1,'linewidth',3);
    plot(tim,100*(PTt/P-1),'--k','linewidth',2);
    
    
    %legend('M','All M','B','all B')
    xlabel('Time, maturity');
    ylabel('Percent');
    text(4.6,100*(1-theta/R)*Mt1_noB(5),'All M','color','k','horizontalalignment','r','fontsize', font_size);
    text(5.2,100*(1-theta/R)*Mt1(6),'M','color',gray1,'fontsize', font_size);
    text(7.5,-1,'B','color','k','fontsize',font_size);
    text(-0.1,2.1,'log(P_t)','color','k','horizontalalignment','r','fontsize',font_size);
    axis([-2.5 9.5 -8 4]);
    print -depsc2 geom_example_3.eps
    
end;

%%%%%%%%%%%%%%%%%%

% from simple_debt_plot

% Figure 4.1 p. 82 

if allplots;
    x = load('data/ngdppot.csv');%1949-2031:10
    yp_yrq = x(:,1);
    yp_moq = x(:,2);
    yp_dyq = x(:,3);
    ypq = x(:,4);
    %convert to annual
    yp = filter([1/4 1/4 1/4 1/4]',1,ypq);
    yp = yp(4:4:end);
    yp_yr = yp_yrq(1:4:end);
    
    x = load('data/FYGFDPUB.csv');%1939-2019
    B_yr = x(:,1);
    B = x(:,4);
    
    x = load('data/fygfdpun.csv'); % 1970:1:2021:3 in millions
    B_yrq = x(:,1);
    B_moq = x(:,2);
    B_dyq = x(:,3);
    Bq = x(:,4);
    
    x = load('data/usrec.csv'); % monhtly through 2021:11
    rec_yr = x(:,1);
    rec_mo = x(:,2);
    rec_dy = x(:,3);
    rec = x(:,4);
    
    x = load('data/fyfsd.csv');%1901-2021 9/30
    sg_yr = x(:,1);
    sg_mo = x(:,2);
    sg_dy = x(:,3);
    sg = x(:,4);
    
    x = load('data/CPIAUCSL.csv'); %1947:1 - 2021:11
    infl_yr = x(13:end,1);
    infl_mo = x(13:end,2);
    infl_dy = x(13:end,3);
    %infl = x(:,4); % pct change from year ago old
    infl = (x(13:end,4)./x(1:end-12,4)-1)*100;
    
    x = load('data/A091RC1Q027SBEA.csv');%1947:1-2021:7 new data is quarterly. old is annual.
    int_cost_yrq = x(:,1);
    int_cost_moq = x(:,2);
    int_cost_dyq = x(:,1);
    int_costq = x(:,4);
    int_cost = filter([1/4 1/4 1/4 1/4]',1,int_costq);
    int_cost = int_cost(4:4:end);
    int_cost_yr = int_cost_yrq(1:4:end);
    %x = load('data/M318501A027NBEA.csv');%1952-2016 identical to omb series but missing last year
    %sg_NIPA_yr = x(:,1);
    %sg_NIPA = x(:,4);
    
    x = load('data/GDPA.csv'); %1929-2020
    y_yr = x(:,1);
    y_mo = x(:,2);
    y_dy = x(:,3);
    y = x(:,4);
    
    x = load('data/GDP.csv'); %47:1 20201:7 billions
    yq_yr = x(:,1);
    yq_mo = x(:,2);
    yq_dy = x(:,3);
    yq = x(:,4);
    
    x = load('data/unrate.csv');%1948:1-2021:11
    u_yr = x(:,1);
    u_mo = x(:,2);
    u_dy = x(:,3);
    unrate = x(:,4);
    
    x = load('data/hall_sargent_sgy.txt');
    yr_os = x(:,1);
    osy_a = x(:,2);
    beap = x(:,3);
    beatot = x(:,4);
    vt = x(:,5);
    
    load Data_Recessions
    Recessions = [Recessions;[datenum(2020,2, 28) datenum(2020,4,30)]];
    
    % divided by actual GDP. Simpler and not much different.
    sg_y = 100*(sg(find(sg_yr==1947):end)/1000)./[y(find(y_yr==1947):end); yq(end)]; %added 2021 manually
    s_y = 100*(sg(find(sg_yr==1947):end)/1000 + [int_cost;int_costq(end)])./[y(find(y_yr==1947):end); yq(end)];
    B_y = 100*B./y(find(y_yr==1939):end-1);
    B_yq = 100*Bq./yq(find((yq_yr==1970)&(yq_mo==1)):end)/1000;
    
    figure;
    set(gca,'box','on')
    hold on
    plot(datenum((1947:2022)',06,01),0*datenum((1947:2022)',06,01),'-k');
    plot(datenum((1947:2021)',06,01),sg_y,'color',gray1,'linewidth',2);
    plot(datenum((1947:2021)',06,01),s_y,'color','k','linewidth',2);
    plot(datenum(u_yr,u_mo,u_dy),-unrate-5,'-k','linewidth',2);
    
    text(datenum(1983,1,1),-2,'Surplus','color',gray1,'fontsize',font_size);
    text(datenum(1978,1,1),5.5,'Primary surplus','color','k','fontsize',font_size);
    text(datenum(1952,1,1),-15,'- Unemployment (right)','color','k','fontsize',font_size);
    hBands = recessionplot('recessions',Recessions);
    %set(hBands,'FaceAlpha',0.01)
    axis([datenum(1945,1,1) datenum(2023,1,1) -20 7]);
    datetick('x','keeplimits')
    xlabel('Date')
    ylabel('Percent');
    
    set(gca,'box','off') % find a better way to remove right hand side ticks
    ax1 = gca;
    ax1_pos = get(ax1,'Position'); % position of first axes
    ax2 = axes('Position',ax1_pos,...
        'XAxisLocation','top',...
        'YAxisLocation','right',...
        'XTickLabel',{},...
        'Xtick',{},...
        'Color','none');
    line(datenum(u_yr,u_mo,u_dy),-unrate,'color','k','linewidth',2);
    axis([datenum(1945,1,1) datenum(2023,1,1) -15 12]);
    
    %print -dpdf simple_debt_graph_1.pdf
    % I used pdf because recessionplot hid the other series in eps graphs. But
    % that problem mysteriously went away. pdf requires trim and clip in the
    % includegraphics, or to figure out how to get matlab not to put huge white
    % space around the graph. includegraphics will also choose the pdf version
    % if it is available, so delete it from the file if you don't want it.
    print -depsc2 simple_debt_graph_1.eps
    
    % Figure 4.2 p. 84 
    
    figure;
    plot(datenum(infl_yr,infl_mo,infl_dy),infl+5,'-k','linewidth',2);
    hold on
    plot(datenum((1947:2022)',06,01),0*datenum((1947:2022)',06,01),'-k');
    plot(datenum((1947:2021)',06,01),s_y,'-k','linewidth',2);
    hBands = recessionplot('recessions',Recessions);
    text(datenum(1945,1,1),-6,'Debt/GDP (right)','color',gray1,'fontsize',font_size);
    text(datenum(1970,1,1),4,'Primary surplus','color','k','fontsize',font_size);
    text(datenum(1984,1,1),15,'Inflation + 5','color','k','fontsize',font_size);
    axis([datenum(1945,1,1) datenum(2023,1,1)  -13 20]);
    
    datetick('x','keeplimits')
    xlabel('Date')
    ylabel('Percent');
    
    set(gca,'box','off') % find a better way to remove right hand side ticks
    ax1 = gca;
    ax1_pos = get(ax1,'Position'); % position of first axes
    ax2 = axes('Position',ax1_pos,...
        'XAxisLocation','top',...
        'YAxisLocation','right',...
        'XTickLabel',{},...
        'Xtick',{},...
        'Color','none');
    line(datenum((1945:1970)',06,01),B_y(find(B_yr==1945):find(B_yr==1970)),...
        'Parent',ax2,'color',gray1,'LineStyle','--','linewidth',3);
    %line(datenum((1947:2018)',06,01),vt,'Parent',ax2,'color','b','linewidth',2);
    line(datenum(B_yrq,B_moq,B_dyq),B_yq,'Parent',ax2','color',gray1,...
        'LineStyle','--','linewidth',3)
    ylabel('Debt/GDP');
    
    axis([datenum(1945,1,1) datenum(2023,1,1)  10 110]);
    print -depsc2 simple_debt_graph_2.eps
    
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% from inflation decomposition/programs_and_data/ftmp_model_final
 
%Figure 5.5. p. 197

if allplots; 
    % [s v u]_t+1' = A [s v u]_t + B eps_t+1
    rho = 0.99; 
    H = 100;
    
    alph_1 = 0.1;
    rhos_1 = 0.7;
    b_s_1 = 1;
    yt=simple_model(alph_1,rhos_1,rho,b_s_1,H);
    yt2=simple_model(alph_1,rhos_1,rho,0,H);
    
    fig_1 = figure;
    %colors = get(fig_1,'defaultaxescolororder');
    plot((0:H-1)',yt(:,1:2), '-k','linewidth',2);
    hold on;
    plot((0:H-1)',yt(:,3),'-','color',gray1,'linewidth',2);
    plot((0:H-1)',yt2(:,1:2),'--k','linewidth',2)
    plot((0:H-1)',0*(0:H-1)','-k')
    axis([0 20 -1.1 2.2]);
    
    text(8,0.3,'s','color','k','fontsize',font_size);
    text(7,1.8,'v','color','k','fontsize',font_size);
    text(8,-0.2,'u','color',gray1,'fontsize',font_size);
    xlabel('Time')
    print -depsc2 simple_example_ir.eps
    
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% From nk_vs_ftpl_simple
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% standard NK model, monetary shock.

% Figure 16.1 p. 359
if allplots;
    T = 50; 

    phink = 1.25;
    rhonk = 0.8;
    pitnk = zeros(T,1);
    pitnk1 = zeros(T,1);
    pitnk2 = zeros(T,1);
    pitnk3 = zeros(T,1);
    vtnk = zeros(T,1);
    itnk = zeros(T,1);
    
    eitnk = zeros(T,1);
    eitnk(2) = 1;
    
    diffpi = 0.12; % amount to shift pi(1) for alternative equilibria.
    
    
    for t = 1:T-1;
        pitnk(t) = -1/(phink-rhonk)* vtnk(t);
        vtnk(t+1) = rhonk*vtnk(t) + eitnk(t+1);
        itnk(t) = phink*pitnk(t) + vtnk(t);
    end;
    pitnk(T) = -1/(phink-rhonk)* vtnk(T);
    itnk(T) = phink*pitnk(T) + vtnk(T);
    
    pitnk1(2) = -1/(phink-rhonk)*eitnk(2)+diffpi;
    pitnk2(2) = -1/(phink-rhonk)*eitnk(2)-diffpi;
    
    for t = 3:T;
        pitnk1(t) = phink*pitnk1(t-1) + vtnk(t-1);
        pitnk2(t) = phink*pitnk2(t-1) + vtnk(t-1);
        pitnk3(t) = phink*pitnk3(t-1) + vtnk(t-1);
    end;
    
    pitnk1(1) = NaN;
    pitnk2(1) = NaN;
    pitnk3(1) = NaN;
    
    
    figure;
    box on;
    hold on;
    plot((0:T-1)',pitnk1,'-','color',gray1,'linewidth',2);
    plot((0:T-1)',pitnk2,'-','color',gray1,'linewidth',2);
    %plot((0:T-1)',pitnk3,'--r','linewidth',1.5);
    
    plot(1,pitnk1(2),'^k');
    plot(1,pitnk2(2),'^k');
    %plot(1,pitnk3(2),'^b');
    
    plot((0:T-1)',pitnk,'-k','linewidth',2);
    
    plot((0:T-1)',itnk,'--k','linewidth',2);
    plot((0:T-1)',vtnk,'-','color',gray1,'linewidth',1.5);
    text(7,-0.7,'\pi_t','color','k','fontsize',font_size)
    text(2,-1.2,'i_t','color','k','fontsize',font_size,'horizontalalignment','r')
    text(2,0.6,'u_t','color',gray1,'fontsize',font_size)
    text(6,-1.3,'Multiple \pi_t','color',gray1,'fontsize',font_size)
    plot((0:T-1)',0*pitnk,'-k')
    
    axis([0 10 -2.5 1.1 ])
    xlabel('Time')
    ylabel('Percent')
    
    
    print -depsc2 NK_only.eps
    
end;


%%%%%%%%%%%%%%%
% copied from simple_ir.m
%%%%%%%%%%%%%%%


% Figure 17.1 p. 393 

if allplots;
    
    % OK phi = 0 i step function
    sig = 1;
    kv = [1/2 1];
    tim = (-3:20)';
    it = [zeros(3,1);ones(21,1)];
    pit_ok = 0*it*(1+0*kv);
    
    for t = 1:(size(it,1)-1);
        pit_ok(t+1,:) = -1./(1./(sig.*kv)).*it(t) + (1+1./(sig.*kv))./(1./(sig.*kv)).*pit_ok(t,:);
    end;
    
    figure
    hold on
    plot(tim+1,[it+0.03],'--k','linewidth',2);
    plot(tim+1,[pit_ok(:,2)-0.03 ],'-','color',gray1,'linewidth',2);
    plot(tim+1,[pit_ok(:,1)-0.03],'-k','linewidth',2);
    axis([-1.2 5 -6 1.5]);
    xlabel('Time')
    ylabel('Percent');
    text(0.5,1,'Interest i','fontsize',font_size,'color','k','horizontalalignment','r');
    text(tim(6)+0.3+1,pit_ok(6,1),'Inflation \pi','fontsize',font_size,'color','k')
    text(tim(6)-0.2+1,pit_ok(6,2),'\pi, less sticky','fontsize',font_size,'color',gray1,'horizontalalignment','r')
    
    
    print -depsc2 simple_ir_eps.eps
    
end

%%%%%%%%%%%%%%

% Figure 17.2 p. 394 

if allplots;
    sig = 1; 
    % NK constant rate
    kv = [1/2 1.5];
    tim = (-3:20)';
    it = [zeros(3,1);ones(21,1)];
    
    pit = 0*it*(1+0*kv);
    for t = 1:(size(it,1)-1);
        pit(t+1,:) = 1./(1+1./(sig.*kv))*it(t) + (1./(sig.*kv))./(1+1./(sig.*kv)).*pit(t,:);
    end;
    
    deltat = ((1./(sig.*kv(1)))./(1+1./(sig.*kv(1)))).^tim;
    
    figure
    hold on
    plot(tim+1,[it ],'--k','linewidth',2);
    
    hold on
    %plot(tim,pit(:,1)*ones(1,5)+deltat*[-.3  .3 1 .8 1.2],'--sm','linewidth',1.5);
    plot(tim(4:end)+1,pit(4:end,1)*ones(1,3)+deltat(4:end,:)*[-.4  .4 1.2],'-','color',gray1,'linewidth',2);
    plot(tim+1,[pit(:,2) ],'-','color','k','linewidth',2);
    plot(tim+1,[pit(:,1)],'-kv','linewidth',2);
    
    xlabel('Time')
    ylabel('Percent');
    text(-0.4+1,0.8,'Interest i','fontsize',font_size,'color','k','horizontalalignment','r','backgroundcolor','w');
    text(tim(6)+1-0.05,pit(6,1),'Inflation \pi','fontsize',font_size,'color','k','backgroundcolor','w');
    text(1+1-0.1,0,'\pi, multiple equilibria','color',gray1,'fontsize',font_size,'backgroundcolor','w');
    text(tim(6)+1,pit(6,2)+0.01,'\pi, less sticky','fontsize',font_size,'color','k',...
        'horizontalalignment','r','backgroundcolor','w')
    axis([-2+1 5+1 -0.4 1.2]);
    
    print -depsc2 simple_ir_NK.eps;
end;

%%%%%%%%%%%%%%%%

% NK response to vi shocks. 

% Figure 17.3 p. 398 

if allplots;
    
    sig = 1;
    kap = 1/2;
    tim = (-5:20)';
    T1 = 6;
    vi0 = -1/2;
    phi = 1.5;
    
    % now with transitory
    rhov = [ 1 0.9 1/(1+sig*kap) 0.3];
    N = size(rhov,2);
    Tt = size(tim,1)-T1+1;
    vit = vi0*[zeros(T1-1,N);(ones(Tt,1)*rhov).^((0:size(tim,1)-T1)'*ones(1,N))];
    pit = -(ones(size(vit,1),1)*(sig*kap ./ (1-rhov + (phi-rhov)*sig*kap))).*vit;
    it = (ones(size(vit,1),1)*((1-rhov*(1+sig*kap))./(1-rhov+(phi-rhov)*sig*kap))).*vit;
    
    
    % Figure 17.3 p. 398 
    
    figure;
    box on
    set(0,'DefaultAxesTitleFontWeight','normal');
    
    for indx = 1:N;
        subplot(2,2,indx)
        box on
        hold on
        %plot(tim+1,0*tim,'-k')
        plot(tim+1,[it(:,indx)./pit(T1,indx) ]+0.02,'--k','linewidth',2);
        plot(tim+1,[pit(:,indx)./pit(T1,indx)]-0.02,'-k','linewidth',2);
        plot(tim+1,vit(:,indx)./pit(T1,indx)-0.02,'-','color',gray1,'linewidth',2);
        
        xlabel('Time')
        ylabel('Percent');
        if ~(indx == 3);
            title(['\eta = ' num2str(rhov(indx))], 'fontsize',font_size);
        else
            title(['\eta = 1/(1+\sigma\kappa) = 2/3'], 'fontsize',font_size);
        end
        if indx == 1;
            text(2,-0.3,'u_i','color',gray1,'fontsize',font_size)
            text(1,0.5,'\pi','color','k','fontsize',font_size)
            text(0,0.7,'i','color','k','fontsize',font_size);
        end
        if indx == 2;
            text(0+1,-0.5,'u_i','color',gray1,'fontsize',font_size)
            text(-0.9+1,0.9,'\pi','color','k','fontsize',font_size)
            text(2+1,0.4,'i','color','k','fontsize',font_size);
        end
        if indx == 3;
            text(2,-0.7,'u_i','color',gray1,'fontsize',font_size)
            text(1.8,1,'\pi','color','k','fontsize',font_size)
            text(3,-0.2,'i','color','k','fontsize',font_size);
        end
        
        if indx == 4;
            text(2.2,-1,'u_i','color',gray1,'fontsize',font_size)
            text(1.3,1,'\pi','color','k','fontsize',font_size)
            text(1,-0.6,'i','color','k','fontsize',font_size);
        end
        
        %text(-0.5,1,'Interest i','fontsize',20,'color',default_color(1,:),'horizontalalignment','r');
        %text(tim(6)+.2,pit(6,1),'Inflation \pi','fontsize',20,'color',default_color(2,:),'backgroundcolor','w');
        axis([-2+1 5+1 -1.2 1.2]);
    end;
    
    print -depsc2 simple_ir_NK_phi.eps;
    
end;

%%%%%%%%%%%%%%%%%%%
%from simple_ir

% OK with step v, phi > 1 
% shock at 1 for book
 
% Figure 17.4 p. 400

if allplots
    tim = (-3:20)';
    it = [zeros(3,1);ones(21,1)];
    sig = 1;
    vit = [zeros(3,1);ones(21,1)];
    phi = 1.5;
    k = 1/2;
    it_ok = 0*vit;
    pit_ok = 0*vit;
    for t = 1:(size(it,1)-1);
        pit_ok(t+1,:) = -sig*k./(1+sig*k*phi).*vit(t) + (1+sig*k)/(1+ sig*k*phi).*pit_ok(t,:);
    end;
    it_ok = vit + phi*pit_ok;
    
    figure;
    plot(tim+1,vit+0.025,'-','color',gray1,'linewidth',2);
    hold on
    plot(tim+1,pit_ok-0.025,'-k','linewidth',2);
    plot(tim+1,it_ok,'--k','linewidth',2);
    axis([-3+1,15,-2 1.2]);
    text(6+1,vit(6+4)-0.2,'Disturbance u_i','color',gray1,'fontsize',font_size,'backgroundcolor','w');
    text(2+1+1,it_ok(2+4),'Interest i','color','k','fontsize',font_size,'backgroundcolor','w');
    text(3+1-1,pit_ok(3+4),'Inflation \pi','color','k','fontsize',font_size,'backgroundcolor','w','horizontalalignment','r');
    %legend('vi','\pi','i');
    xlabel('Time')
    ylabel('Percent')
    print -depsc2 old_k_ir_withphi.eps;
    
end;

%%%%%%%%%%%%%%
%from nk_ir

%Figure 17.5 p. 402 

if allplots;
    
    b = 0.97; % beta
    k = 0.2; % kappa
    s = 1; % sigma
    xi =0.1; % xi interest elasticity of money demand
    mamax = 50; % how many ir to calculate
    maxt = 50; % was 10 % how far forward to run irs
    itargt = [zeros(10,1);1;ones(maxt,1)]; %interest target
    tim = (-10:maxt)'; % must be symmetric!
    t0 = 11; % index of t = 0
    xlimits = [-5 7]; % range for x axis of impulse response plots
    
    %simple nk model
    
    [pit,xt,pit_unex,xt_unex,r1,r2,l1,l2,l3,phi,ma] =...
        fn_find_ir(itargt,0,t0,b,k,s,xi,0,0,mamax);
    % shifted shock to 1 for book
    figure
    plot(tim+1,itargt,'--k','linewidth',2);
    hold on
    plot(tim+1,pit,'-k','linewidth',2);
    plot(tim+1,xt,'-','color',gray1,'linewidth',2);
    plot(tim+1,0*tim,'-k')
    plot(tim+1,pit_unex,'--k','linewidth',2);
    plot(tim+1,xt_unex,'--','color',gray1,'linewidth',2);
    plot(tim+1,pit,'-k','linewidth',2);
    plot(tim+1,xt,'-','color',gray1,'linewidth',2);
    text(2+1,itargt(find(tim==2)),'Interest rate i','color', 'k','backgroundcolor','w','fontsize',font_size);
    text(2+2,pit(find(tim==2)),'Inflation \pi','color', 'k','fontsize',font_size);
    text(2+1.5,xt(find(tim==2)),'Output gap x','color', gray1,'fontsize',font_size);
    text(0.4,-0.2,'Unexpected','color', gray1,'fontsize',font_size);
    
    axis([xlimits+1  -inf 1.1]);
    %legend('i','\pi','x','location','NorthWest');
    xlabel('Time');
    ylabel('Percent')
    print -depsc2 nk_ir_1.eps;
    
    %%%%%%%%%
    
    % delta shocks in nk model with short term debt
    % show some alternative solutions, which add geometric decay
    % find s = 0 solution
    
    Tb = 0;
    [pit,xt,pit_unex,xt_unex,r1,r2,l1,l2,l3,phi,ma] = fn_find_ir(itargt,0,t0,b,k,s,xi,0,0,mamax);
    global_pars = {tim, l1, l2, pit, xt, t0, s, b, k, Tb,1,1};
    ds0 = fzero(@f_finds,0.5);
    
    % find other solutions
    
    d0 = [ ds0 1.2-pit(t0) 1-pit(t0) 0-pit(t0) -0.5-pit(t0) ];
    [bigpit, bigxt, dsbig] = fn_add_delta(tim,Tb,l1,l2,d0,pit,xt,b,s,k,t0,1,1);
    [pit0, xt0, ds0] = fn_add_delta(tim,Tb,l1,l2,0,pit,xt,b,s,k,t0,1,1);
    % 0 separate to give different color and fdlabel position
    
    % Figure 17.6 p. 403 
    
    figure
    hold on
    axis([-2 6  -0.6 1.3]);
    plot(tim+1,itargt+0.02,'--k','linewidth',2);
    plot(tim+1,0*tim,'-k');
    %legend('i','\pi','x');
    [sortpi0, indxsort] = sort(-bigpit(t0,:)');
    lbls = ['A'; 'B'; 'C'; 'D'; 'E'; 'F'; 'G'];
    letterx = 0.8;
    for indx = 1:size(bigpit,2);
        text(letterx,bigpit(t0,indxsort(indx)),[lbls(indx,:) ',\Deltas=' num2str(dsbig(indxsort(indx)),'%3.2f')],...
            'HorizontalAlignment','right','color',gray1,'fontsize',font_size,'backgroundcolor','w')
    end;
    text(letterx,pit0(t0),['\delta=0',',\Deltas=' num2str(ds0,'%3.2f')],...
        'HorizontalAlignment','right','color','k','fontsize',font_size,'backgroundcolor','w')
    
    
    plot(tim(t0:end)+1,pit(t0:end),'-k','linewidth',2);
    plot(tim+1,bigpit,'-','color',gray1,'linewidth',2);
    
    %axis([xlimits  -0.6 1.3]);
    
    xlabel('Time');
    ylabel('Percent')
    print -depsc2 nk_ir_2.eps;
end;

% %%%%%%%%%%%%%
% from graphics\NK_fiscal

% Figure 17.7 p. 406 

if allplots
    T = 50;
    
    sig = 1;
    kap = 0.25;
    eta = 0.6;
    bet = 0.95;
    phi = 1.2;
    pi1 = [-1 -0.5  0 0.5];
    
    lam1 = ((1+bet+sig*kap)+ ((1+bet+sig*kap)^2-4*bet)^0.5)/2;
    lam2 = ((1+bet+sig*kap)- ((1+bet+sig*kap)^2-4*bet)^0.5)/2;
    lam1i = lam1^(-1);
    
    delt = pi1 - sig*kap/(lam1-lam2)*lam2/(1-lam2*eta);
    
    tim = (0:1:T-1)';
    
    pit = zeros(T,1);
    pit(2) = sig*kap/(lam1-lam2)*lam2/(1-lam2*eta) ; % t=1
    pit(3) = sig*kap/(lam1-lam2)*(1/(1-lam2*eta)) ;
    for indx = 4:T;
        pit(indx) = sig*kap/(lam1-lam2)*...
            (eta^(indx-3)/(1-lam2*eta) + lam1i*(eta^(indx-3)-lam1i^(indx-3))/(eta-lam1i) );
    end;
    
    pim = [pit*(1+0*pi1) + [0*delt;(lam1i.^((0:T-2)')).*delt]];
    
    it = [0; eta.^(0:1:T-2)'];
    um = it*(1+0*pi1) - phi*pim;
    
    rterm = sum(it(2:end-1,:)-pim(3:end,:));
    sterm = rterm-pim(2,:);
    
    figure; % 4 panel plot
    set(0,'DefaultAxesTitleFontWeight','normal');
    for indx = 1:4;
        subplot(2,2,indx);
        box on;
        hold on;
        plot(tim,pim(:,indx),'-k','linewidth',2);
        if indx == 1;
            text(1.8,-0.7,'\pi','color','k','fontsize',font_size)
            text(1,0.7,'i','color','k','fontsize',font_size);
            text(2.4,1,'u','color',gray1,'fontsize',font_size)
        end
        if indx == 2;
            text(1.2,-0.5,'\pi','color','k','fontsize',font_size)
            text(1,0.7,'i','color','k','fontsize',font_size);
            text(1.7,1.2,'u','color',gray1,'fontsize',font_size)
        end
        if indx == 3;
            text(1,-0.1,'\pi','color','k','fontsize',font_size)
            text(2,0.8,'i','color','k','fontsize',font_size);
            text(4,-0.3,'u','color',gray1,'fontsize',font_size)
        end
        if indx == 4;
            text(4,0.7,'\pi','color','k','fontsize',font_size)
            text(1,1.2,'i','color','k','fontsize',font_size);
            text(3,-0.4,'u','color',gray1,'fontsize',font_size)
        end
        plot(tim,um(:,indx),'-','color',gray1,'linewidth',2);
        plot(tim,it,'--k','linewidth',2);
        plot(tim,0*tim,'-k')
        xlabel('Time');
        ylabel('Percent');
        title(['\Sigma s = ' num2str(sterm(indx),'%4.2f')],'fontsize',font_size)
        axis([ 0 6 -1 1.5])
    end
    print -depsc2 nk_fiscal_1.eps
end

%%%%%%%%%%%%%%%%%%%%
% From three_equation_response.m

% Figure 17.8 p. 409 

if allplots;
    bet = 0.95;
    k = 1/2;
    sig = 1;
    phi = 1.5;
    
    % ***********************
    % response to m policy shock
    % **********************
    
    lp = (1 + bet + k*sig + ((1 + bet + k*sig)^2 - 4 * 1 * bet * (1+phi*k*sig))^0.5 )/(2*(1+k*sig*phi));
    ln = (1 + bet + k*sig - ((1 + bet + k*sig)^2 - 4 * 1 * bet * (1+phi*k*sig))^0.5 )/(2*(1+k*sig*phi));
    % these are lambdas of lag operator approach, 1/lambdas of matrix approach
    
    l1 = (1 + bet + k*sig + ((1 + bet + k*sig)^2 - 4 * bet )^0.5 )/2;
    l2 = (1 + bet + k*sig - ((1 + bet + k*sig)^2 - 4 * bet )^0.5 )/2;
    % these are from mm paper, lambdas of the i fixe approach
    
    rhov = [1 0.8 1/l1 0.3]';
    T = 10;
    
    figure;
    set(0,'DefaultAxesTitleFontWeight','normal');
    for rhoi = 1:size(rhov,1);
        rho = rhov(rhoi);
        ihatt = zeros(T,1);
        ihatt(3) = -1; % shock
        ihatt(4:T) = -rho.^((1:T-3)');
        pit = -k*sig/(1+k*sig*phi)*1/((1-lp*rho)*(1-ln*rho))*ihatt;
        it = phi*pit + ihatt;
        
        subplot(2,2,rhoi);
        box on
        hold on
        plot((1-3:T-3)'+1,pit+0.05,'-k','linewidth',2)
        plot((1-3:T-3)'+1,it,'--','color','k','linewidth',2)
        plot((1-3:T-3)'+1,ihatt-0.05,'-','color',gray1,'linewidth',2)
        text(4-3+1,pit(4),'\pi','color','k','fontsize',font_size,'backgroundcolor','w')
        text(3-3+1,ihatt(3),'u_i','color',gray1,'fontsize',font_size,'backgroundcolor','w')
        text(3-3+1,it(3),'i','color','k','fontsize',font_size,'backgroundcolor','w')
        %legend('\pi','i','v^i')
        xlabel('Time');
        ylabel('Percent')
        if ~(rhoi == 3);
            title(['\eta = ' num2str(rho)],'fontsize',font_size);
        end;
        if (rhoi == 3);
            title(['\eta = ' num2str(rho,'%4.2f')],'fontsize',font_size);
        end;
        if rhoi == 1;
            axis([-2+1 6+1 -1.25 2.3]);
        end;
        if rhoi == 2;
            axis([-2+1 6+1 -1.25 1.5]);
        end;
        if rhoi == 3;
            axis([-2+1 6+1 -1.25 0.8]);
        end;
        if rhoi == 4;
            axis([-2+1 6+1 -1.25 0.6]);
        end;
    end;
    
    print -depsc2 three_equation_response.eps
end

%%%%%%%%%%%%%%%%%%
% from money_in_utility
 
% Figure 19.1 p. 436

if allplots;
    gam = 2;
    delt = 0.2;
    mu = 0.2;
    th = 1/100;
    newi = 0; % 1: new version with i; 0: old version with i/(1+i)
    
    m_sat = 1/3; % M/PY at which um goes to zero
    
    % i timing %
    
    pym_new = (1/th*((1+delt)*(1+mu)-1)).^(1/gam);
    pym_0_new = (0:0.1:3*floor(pym_new))';
    pym_1_new = (1+th*pym_0_new.^(gam))./(1+th*pym_new.^(gam)).*pym_0_new;
    
    % i/(1+i) timing
    
    pym = (1/th * (1-1/((1+delt)*(1+mu))))^(1/gam);
    pym_0 = (0:0.1:2*floor(pym))';
    pym_1 = (1-th*pym^(gam))./(1-th*pym_0.^(gam)).*pym_0;
    
    % 1/1+i and msat
    
    pym_msat = (1/th * (1-1/((1+delt)*(1+mu)))+m_sat^(-gam))^(1/gam);
    pym_msat_0 = (0:0.1:2*floor(pym_msat))';
    mpy_msat_1 = (1-th*(pym_msat_0.^(gam)-m_sat^(-gam)))./...
        (1-th*(pym_msat^(gam)-m_sat^(-gam)))./pym_msat_0;
    mpy_msat_1 = (pym_msat_0>1/m_sat).*mpy_msat_1 + ...
        (pym_msat_0<=1/m_sat).*((1+delt)*(1+mu))./pym_msat_0;
    pym_msat_1 = 1./mpy_msat_1;
    
    figure
    box on
    hold on
    plot(pym_0,pym_1,'-k','linewidth',2);
    %plot(pym_0_new,pym_1_new,'--r','linewidth',2);
    plot([pym pym]',[0 pym],'--k','linewidth',1.5)
    plot([0 pym]',[pym pym],'--k','linewidth',1.5)
    text(pym+0.2,0.5,'PY/M','fontsize',font_size)
    plot(pym_0,pym_0,'-k','linewidth',1)
    xlabel('P_tY/M_t');
    ylabel('P_{t+1}Y/M_{t+1}');
    
    axis([0 8 0 8]);
    print -depsc2 money_in_utility_phase.eps
    
end;

%%%%%%%%%%%%%%%%
% New version of interest, inflation and fed funds plot


% Figure 20.1 p. 459

if allplots;
    x = load('data/GS10.csv');
    y10yr = x(:,1);
    y10mo = x(:,2) ;
    y10dy = x(:,3) ;
    y10 = x(:,4);
    
    x = load('data/fedfunds.csv');
    ffyr = x(:,1);
    ffmo = x(:,2) ;
    ffdy = x(:,3) ;
    ff = x(:,4);
    
    x = load('data/cpilfesl.csv');
    cpiyr = x(13:end,1);
    cpimo = x(13:end,2) ;
    cpidy = x(13:end,3) ;
    cpi = (x(13:end,4)./x(1:end-12,4)-1)*100;
    
    x = load('data/wtfsrfa.csv');
    astyr = x(:,1);
    astmo = x(:,2);
    astdy = x(:,3);
    ast = x(:,4);
    
    load Data_Recessions
    Recessions = [Recessions;[datenum(2020,2, 28) datenum(2020,4,30)]];
    
    figure
    subplot(2,1,1)
    hold on;
    box on
    %plot(datenum(y10yr,y10mo,y10dy),y10,'-','color',gray1,'linewidth',2);
    plot(datenum(ffyr,ffmo,ffdy),ff,'-','color','k','linewidth',2);
    plot(datenum(cpiyr,cpimo,cpidy),cpi,'-','color',gray1,'linewidth',2);
    hBands = recessionplot('recessions',Recessions);
    axis([datenum(2000,1,1) datenum(2022,1,1) -inf 7]);
    text(datenum(2001,6,1),5,'Fed funds','color','k','fontsize',font_size);
    text(datenum(2012,1,1),3,'Core cpi','color',gray1,'fontsize',font_size);
    ylabel('Percent');
    datetick('x','keeplimits')
    
    subplot(2,1,2)
    hold on;
    box on;
    %plot(datenum(y10yr,y10mo,y10dy),y10,'-','color',gray1,'linewidth',2);
    plot(datenum(y10yr,y10mo,y10dy),y10,'-','color',gray1,'linewidth',2);
    hBands = recessionplot('recessions',Recessions);
    text(datenum(2003,1,1),6,'10 yr. treasury','color',gray1,'fontsize',font_size);
    text(datenum(2016,1,1),4,'Fed assets','color','k','fontsize',font_size);
    
    axis([datenum(2000,1,1) datenum(2022,1,1) -inf 7]);
    ylabel('Percent')
    datetick('x','keeplimits')
    
    box off
    ax=gca;
    axes('position',ax.Position,'box','on','ytick',[],'xtick',[],'color','none')
    %set(gca,'box','off') % find a better way to remove right hand side ticks
    ax1 = gca;
    ax1_pos = get(ax1,'Position'); % position of first axes
    ax2 = axes('Position',ax1_pos,...
        'XAxisLocation','bottom',...
        'YAxisLocation','right',...
        'XTickLabel',{},...
        'Xtick',{},...
        'Color','none');
    line(datenum(astyr,astmo,astdy),ast/10^6,'color','k','linewidth',2);
    axis([datenum(2000,1,1) datenum(2022,1,1) -inf inf]);
    ylabel('$ Trillion');
    datetick('x','keeplimits')
    print -depsc2 tb_core_10y_Reserves.eps
    
end;

%%%%%%%%%
% New Japan plot

% Figire 20.2 p. 459
if allplots
    
    x = load('data/JPNCPICORMINMEI.csv');
    cpiyr = x(:,1);
    cpimo = x(:,2);
    cpidy = x(:,3);
    cpi = x(:,4);
    
    x = load('data/IRSTCB01JPM156N.csv');
    ffyr = x(:,1);
    ffmo = x(:,2);
    ffdy = x(:,3);
    ff = x(:,4);
    
    x = load('data/IRLTLT01JPM156N.csv');
    y10yr = x(:,1);
    y10mo = x(:,2);
    y10dy = x(:,3);
    y10 = x(:,4);
    
    figure
    subplot(2,1,1)
    hold on
    box on
    plot(datenum(cpiyr,cpimo,cpidy),cpi,'-','color',gray1,'linewidth',2);
    plot(datenum(ffyr,ffmo,ffdy),ff,'-','color','k','linewidth',2);
    plot(datenum(ffyr,ffmo,ffdy),0*ff,'-k','linewidth',1);
    text(datenum(1993,6,1),3,'Overnight rate','color','k','fontsize',font_size);
    text(datenum(2004,1,1),-1,'Core CPI','color',gray1,'fontsize',font_size);
    axis([datenum(1992,1,1) datenum(2022,1,1) -2 5]);
    ylabel('Percent');
    datetick('x','keeplimits');
    
    subplot(2,1,2)
    hold on
    box on
    plot(datenum(ffyr,ffmo,ffdy),ff,'-k','linewidth',2);
    plot(datenum(y10yr,y10mo,y10dy),y10,'-','color',gray1,'linewidth',2);
    plot(datenum(ffyr,ffmo,ffdy),0*ff,'-k','linewidth',1);
    text(datenum(1993,1,1),-0.2,'Overnight rate','color','k',...
        'fontsize',font_size,'backgroundcolor','w');
    text(datenum(1996,1,1),4,'10 year rate','color',gray1,'fontsize',font_size);
    
    axis([datenum(1992,1,1) datenum(2022,1,1) -1 6]);
    ylabel('Percent');
    datetick('x','keeplimits');
    print -depsc2 japan_1.eps
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
% From simple_ir 

% Figure 20.3 p. 465

if allplots;
    taylormax = 1; % is it peg or not, or is it max(taylor, 0);
    k = 1/2;
    sig = 1;
    phi = 2;
    pistar = 2;
    vrpre = -2;
    vrpost=0; % vr shocks
    T = 10;
    T2 = 15;
    tim = (1:T2)';
    %pi1 = ([-3 -2 -1 -0.5 0 0.5 1 ])+pistar;
    pi1 = ([ -1 -0.5 0 0.5 ])+pistar;
    
    indx0 = find(pi1==pistar);
    pit = 0*tim*pi1;
    it = 0*pit;
    pit(T,:)=pi1;
    if ~taylormax;
        it(T,:) = phi*pit(T,:) + (1-phi)*pistar;
    end;
    if taylormax;
        it(T,:) = max( [(phi*pit(T,:) + (1-phi)*pistar);0*pit(T,:)] );
    end;
    for j = 1:size(pi1,2);
        for t = T:-1:2;
            if ~taylormax;
                pit(t-1,j) = (1+sig*k)*pit(t,j)+sig*k*vrpre; % work back
                it(t-1,j) = 0;
            end;
            if taylormax;
                pit(t-1,j) = (1+sig*k)/(1+phi*sig*k)*(pit(t,j)-pistar) + ...
                    pistar + sig*k/(1+phi*sig*k)*vrpre; %guess taylor
                it(t-1,j) = phi*pit(t-1,j) + (1-phi)*pistar;
                if it(t-1,j) < 0; %whoops, zero boud
                    pit(t-1,j) = (1+sig*k)*pit(t,j)+sig*k*vrpre; % work back
                    it(t-1,j) = 0;
                end;
            end;
            
        end;
        for t=T+1:T2;
            if ~taylormax
                pit(t,j) = pistar + (1+sig*k*phi)/(1+sig*k)*(pit(t-1,j)-pistar)...
                    -sig*k/(1+sig*k)*vrpost;
                it(t,j) = phi*pit(t,j) + (1-phi)*pistar;
            end;
            if taylormax
                if it(t-1,j) > 0;
                    pit(t,j) = pistar + (1+sig*k*phi)/(1+sig*k)*(pit(t-1,j)-pistar)...
                        -sig*k/(1+sig*k)*vrpost;
                    it(t,j) = max(phi*pit(t,j) + (1-phi)*pistar,0);
                end;
                if it(t-1,j) <= 0; % both phi and vi become 0
                    pit(t,j) = 1/(1+sig*k)*pit(t-1,j)...
                        -sig*k/(1+sig*k)*vrpost;
                    it(t,j) = max(phi*pit(t,j) + (1-phi)*pistar,0);
                end
            end;
        end;
    end;
    
    % Figure 20.3 p. 465
    figure;
    subplot(2,1,1)
    box on
    hold on
    plot(tim,pit,'-','color',gray1,'linewidth',2);
    plot(tim,0*tim,'-k')
    plot(tim,pit(:,indx0),'-k','linewidth',3)
    plot(T,2,'kv','markerfacecolor','k')
    plot([T T], [-10 10], '--k');
    text(T-2.5,-3,['\leftarrow u_r = ' num2str(vrpre)],'fontsize',font_size)
    text(T+0.5,-3,'u_r = 0 \rightarrow','fontsize',font_size)
    text(3,3,'Inflation \pi','color','k', 'fontsize', font_size)
    axis([2 T2 -4 4]);
    %xlabel('Time');
    ylabel('Percent');
    
    subplot(2,1,2)
    box on
    hold on
    plot(tim,0*tim,'-k')
    plot(tim,it,'-','color',gray1,'linewidth',2)
    plot(tim,it(:,indx0),'-k','linewidth',3)
    %plot([0 0], [-10 10], '--k');
    plot([T T], [-10 10], '--k');
    plot(T,2,'kv','markerfacecolor','k')
    text(T-2.5,5,['\leftarrow u_r = ' num2str(vrpre)],'fontsize',font_size)
    text(T+0.5,5,'u_r = 0 \rightarrow','fontsize',font_size,'backgroundcolor','w')
    text(3,4,'Interest rate i', 'color','k','fontsize', font_size)
    axis([2 T2 -1 6]);
    xlabel('Time');
    ylabel('Percent');
    print -depsc2 escape2.eps;
    
    %escape2i is canceled
end;

%%%%%%%%%%%%%%%%
% New plots for covid section 
%%%%%%%%%%%%%%

% Figures 21.1 and 21.2 
if allplots;
    
    x = load('data/cpiaucsl.csv');
    cpi_yr = x(:,1);
    cpi_mo = x(:,2);
    cpi_dy = x(:,3);
    cpi = x(:,4);
    
    x = load('data/cpilfesl.csv');
    cpife_yr = x(:,1);
    cpife_mo = x(:,2);
    cpife_dy = x(:,3);
    cpife = x(:,4);
    
    x = load('data/M2SL.csv');
    M2_yr = x(:,1);
    M2_mo = x(:,2);
    M2_dy = x(:,3);
    M2 = x(:,4);
    
    x = load('data/bogmbase.csv');
    Mb_yr = x(:,1);
    Mb_mo = x(:,2);
    Mb_dy = x(:,3);
    Mb = x(:,4);
    
    x = load('data/FYGFDPUN.csv');
    B_yr = x(:,1);
    B_mo = x(:,2);
    B_dy = x(:,3);
    B = x(:,4);
    
    load Data_Recessions
    Recessions = [Recessions;[datenum(2020,2, 28) datenum(2020,4,30)]];
    
    
    % Figure 21.1 p. 479 
    
    figure;
    hold on
    box on
    plot(datenum(cpi_yr,cpi_mo,cpi_dy),cpi,'-k','linewidth',2);
    plot(datenum(cpife_yr,cpife_mo,cpife_dy),cpife,'-','color',gray1,'linewidth',2);
    plot(datenum(cpi_yr,cpi_mo,cpi_dy),cpi,'kv','linewidth',1);
    plot(datenum(cpife_yr,cpife_mo,cpife_dy),cpife,'ks','color',gray1,'linewidth',1);
    ax = gca;
    ax.XGrid = 'on';
    ax.YGrid = 'off';
    plot(datenum(2021,1,1)*[1;1],[240; 290],'-k')
    text(datenum(2019,1,1),267,'Core CPI','color',gray1,'fontsize',font_size);
    text(datenum(2020,6,1),255,'CPI','color','k','fontsize',font_size);
    axis([datenum(2018,1,1) datenum(2022,1,1) 245 285])
    hBands = recessionplot('recessions',Recessions);
    ylabel('Index, 1982-84 = 100')
    datetick('x','keeplimits')
    print -depsc2 covid_cpi.eps
    
    % Figure 21.2 p. 480 
    
    hold on
    box on
    plot(datenum(M2_yr,M2_mo,M2_dy),M2/1E3,'-','color',gray1,'linewidth',2);
    plot(datenum(B_yr,B_mo,B_dy),B/1E6,'-','color','k','linewidth',2);
    text(datenum(2013,1,1),14,'Debt','fontsize',font_size,'color','k');
    text(datenum(2018,1,1),13,'M2','fontsize',font_size,'color',gray1);
    text(datenum(2014,1,1),7,'M base, right scale','fontsize',font_size,'color','k');
    hBands = recessionplot('recessions',Recessions);
    axis([datenum(2007,1,1) datenum(2022,1,1) 0 23]);
    ylabel('$ Trillion')
    datetick('x','keeplimits')
    
    ax=gca;
    box off
    axes('position',ax.Position,'box','on','ytick',[],'xtick',[],'color','none')
    %set(gca,'box','off') % find a better way to remove right hand side ticks
    ax1 = gca;
    ax1_pos = get(ax1,'Position'); % position of first axes
    ax2 = axes('Position',ax1_pos,...
        'XAxisLocation','bottom',...
        'YAxisLocation','right',...
        'XTickLabel',{},...
        'Xtick',{},...
        'Color','none');
    line(datenum(Mb_yr,Mb_mo,Mb_dy),Mb/1E6,'color','k','linewidth',2);
    ylabel('$ Trillion')
    axis([datenum(2007,1,1) datenum(2022,1,1) 0 10]);
    datetick('x','keeplimits');
    print -depsc2 covid_m_and_b.eps
end;

%%%%%%%%%%%%%%%%%
% From solve_ftmp_sticky_longdebt_new, out of order
%%%%%%%%%%%%%%%%%%%%%%%

% Note: there are three scripts. I converted everything to
% scritp_solve_ftmp_longdebt_new, but it looks in retrospect like I 
% kept the matrix script as one can't take limits in the longdebt script. 
% it's right, but if you ever develop the programs, write a common one that
% uses appropriate matrices for the limiting cases, i.e. no debt, rather
% than 0.01 and 0.99 sort of things

 
% Figure 5.4 p. 127 

if allplots;
    r = 0.01;
    sigma = 1;
    kappa1 = 0.25; %0.25; % use kappa = 100 to verify it reduces to frictionless case
    kappa2 = 99;
    kappa = kappa1;
    thet_ipi = 0;
    rhoi = 0.99;
    rhos = 0.0;
    omeg1 = 0.85;
    omeg2 = 0.9;
    omeg = omeg1;
    
    T = 50;
    R = 1+r;
    bet = 1/R;
    
    shock = [ 1 0 ]'; % shock to ei es  For IR function
    viknown = zeros(T,1); % known in advance vi
    
    figure
    box on
    hold on
    
    omeg = omeg2;
    script_solve_ftmp_longdebt_new;  % script returns xt, pit, vt, st, uit, vt, it, rt
    plot((-2:T-1)',[0;0;pit],'-k','linewidth',2);
    omeg = omeg1;
    kappa = kappa2;
    script_solve_ftmp_longdebt_new; % script returns xt, pit, vt, st, uit, vt, it, rt
    kappa = kappa1;
    plot([0 1 1.2]',...
        [pit(2)+1*(pit(1)-pit(2));...
        pit(2);...
        pit(2)+0.2*(pit(3)-pit(2))],'-k','linewidth',2);
    plot(1,pit(2),'ok');
    plot([1.9 2],[pit(2)+0.9*(pit(3)-pit(2));pit(3)],'-k','linewidth',2);
    
    script_solve_ftmp_longdebt_new; % % script returns xt, pit, vt, st, uit, vt, it, rt
    
    plot((-2:T-1)',[0;0;it],'--k','linewidth',2);
    plot((-2:T-1)',[0;0;xt],'-s','color',gray1,'linewidth',2);
    plot((-2:T-1)',[0;0;pit],'-kv','linewidth',2);
    %plot((0:T-1)',vt,'-*m','linewidth',2);
    plot((-2:T-1)',zeros(T+2,1),'-k');
    
    xlabel('Time')
    ylabel('Percent')
    
    text(0.2,0.8,'i','fontsize',font_size,'color','k')
    text(1.7,-0.4,'\pi ','fontsize',font_size,'color','k')
    text(3.3,-0.3,'\pi, \omega=0.9','fontsize',font_size,'color','k')
    text(2.3,xt(3),'x','fontsize',font_size,'color',gray1)
    text(1.3,-4.5,'\pi, \kappa = \infty','color','k','fontsize',font_size);
    axis([-1 6 -5.5 1.5])
    
    print -depsc2 ftmp_sticky_longdebt_i.eps
    
end;

%%%%%%%%%%%%%

% Figure 5.2 p. 123

if allplots ;  
    r = 0.01;
    sigma = 1;
    kappa1 = 0.5; %0.25; % use kappa = 100 to verify it reduces to frictionless case
    kappa2 = 999; 
    kappa = kappa1; 
    thet_ipi = 0;
    rhoi = 0.99;
    rhos = 0;
    omeg1 = 0.01;
    omeg = omeg1;
    
    T = 50;
    R = 1+r;
    bet = 1/R;
    
      % expected interest rate
    viknown = zeros(T,1); % known in advance vi
    viknown(6:end) = 1; %expected in advance at time T=5.
    shock = [ 0 0 ]';
    script_solve_ftmp_longdebt_new; % % script returns xt, pit, vt, st, uit, vt, it, rt
    
    figure
    box on
    hold on
    
    plot((0:T-1)'-4,[it],'--k','linewidth',2);
    plot((0:T-1)'-4,[xt],'-s','color','k','linewidth',2);
    %plot((-2:T-1)',[0;0;vit],'--k','linewidth',2);
    %plot((0:T-1)',bt,'-*m','linewidth',2);
    plot((0:T-1)'-4,[rt],'-','color',gray1,'linewidth',2);
    plot((0:T-1)'-4,zeros(T,1),'-k');
    plot((0:T-1)'-4,[pit],'-k','linewidth',3)
    %legend('\pi_t','i_t','x_t','r_t');
    
    text(4-3.7,0.7,'i','fontsize',font_size,'color','k')
    text(2,0.8,'\pi','fontsize',font_size,'color','k')
    %text(3.2,vit(4)+0.1,'v^i_t','fontsize',font_size,'color','k')
    text(7.4-4,xt(8)-0.05,'x','fontsize',font_size,'color','k')
    text(7.4-4,rt(8)+0.06,'r','fontsize',font_size,'color',gray1)
    %axis([0 15 -1 1.1])
    axis([-3 6 -0.8 1.1]) 
    
    kappa = kappa/5;
    script_solve_ftmp_longdebt_new; % script returns xt, pit, vit, st, bt, it
    kappa = kappa*5;
    
    plot((0:T-1)'-4,[pit],'-v','color',gray1,'linewidth',2);
    text(3,0.6,'\pi, \kappa = 0.1','color',gray1,'fontsize',font_size)
    
    xlabel('Time')
    ylabel('Percent')
    
    print -depsc2 expected_i_sticky.eps
end;

%%%%%%%%%%
% new, uses longdebt script for everything including this one

% Figure 5.1 p. 121 

if allplots;
    r = 0.01;
    sigma = 1;
    kappa = 0.5; %0.25; % use kappa = 100 to verify it reduces to frictionless case
    thet_ipi = 0;
    rhoi = 0.999;
    rhos = 0.0;
    omeg = 0.01;
    
    T = 50;
    R = 1+r;
    bet = 1/R;
    
    shock = [ 1 0 ]'; % shock to ei es  For IR function
    viknown = zeros(T,1); % known in advance vi
   
    script_solve_ftmp_longdebt_new; % script returns xt, pit, vit, st, bt, it
    
    figure
    box on
    hold on
    plot((-2:T-1)',[0;0;rt],'-','color',gray1,'linewidth',2);
    plot((-2:T-1)',[0;0;it],'--k','linewidth',2);
    plot((-2:T-1)',[0;0;xt],'-s','color','k','linewidth',2);
    plot((-2:T-1)',[0;0;pit],'-k','linewidth',3);
    %plot((0:T-1)',bt,'-*m','linewidth',2);
    plot((-2:T-1)',zeros(T+2,1),'-k');
    
    xlabel('Time')
    ylabel('Percent')
    
    text(0.2,0.8,'i','fontsize',font_size,'color','k')
    text(1.3,pit(2),'\pi','fontsize',font_size,'color','k')
    text(2.5,xt(3),'x','fontsize',font_size,'color','k')
    text(1.4,rt(2)+0.03,'r','fontsize',font_size,'color',gray1)
    %legend('r_t','i_t','x_t','pi_t');
    axis([-1 6 -0.8 1.1])
    
    kappa = kappa/5;
    script_solve_ftmp_longdebt_new; % script returns xt, pit, vit, st, bt, it
    kappa = kappa*5;
    
    plot((-2:T-1)',[0;0;pit],'-v','color',gray1,'linewidth',2);
    text(4,0.8,'\pi, \kappa = 0.1','color',gray1,'fontsize',font_size)
    %plot((-2:T-1)',[0;0;xt],'--k','linewidth',2);
    
    print -depsc2 ftmp_sticky_matrix_i.eps
    
end
 
% Figure 5.3 p. 124

if allplots;
    r = 0.01;
    sigma = 1;
    kappa1 = 0.25; %0.25; % use kappa = 100 to verify it reduces to frictionless case
    kappa2 = 99;
    kappa = kappa1;
    thet_ipi = 0;
    rhoi = 0.99;
    rhos = 0;
    omeg1 = 0.01;
    omeg2 = 0.9;
    omeg = omeg1;
    
    T = 50;
    
    R = 1+r;
    bet = 1/R;
    
    
    viknown = zeros(T,1); % known in advance vi
    
    
    shock = [ 0 -1 ]'; % shock to ei es  For IR function
    script_solve_ftmp_longdebt_new; % % script returns xt, pit, vt, st, uit, vt, it, rt
    
    
    figure
    box on
    hold on
    plot((-2:T-1)',zeros(T+2,1),'-k');
    plot((-2:T-1)',[0;0;it],'--k','linewidth',2);
    plot((-2:T-1)',[0;0;xt],'-s','color',gray1,'linewidth',2);
    %plot((0:T-1)',vit,'--k','linewidth',2);
    %plot((0:T-1)',bt,'-*m','linewidth',2);
    plot((-2:T-1)',[0;0;rt],'-','color','k','linewidth',2);
    plot((-2:T-1)',[0;0;pit],'-k','linewidth',2);
    
    
    xlabel('Time')
    ylabel('Percent')
    
    text(2.2,it(3)-0.04,'i','fontsize',font_size,'color','k')
    text(1.8,pit(3),'\pi','fontsize',font_size,'color','k','horizontalalignment','r')
    %text(2.2,vit(3)-0.05,'v^i_t','fontsize',16','color','k')
    text(2.2,xt(3)+.05,'x','fontsize',font_size,'color',gray1)
    text(2.2,rt(3)-0.04,'r','fontsize',font_size,'color','k')
    %axis([-1 12 -0.7 0.3])
    axis([-1 6 -0.3 0.7])
    
    print -depsc2 ftmp_sticky_matrix_es_nophi.eps  
    
end;

%%%%%%%%%%%
% from Hall_data/fiscal_book_picture, not changed. 

% Figure 7.6 p. 233

if allplots;
    x = load('Hall_data/US_treasury_debt_service_profile_2020.csv');
    year = x(2:end,1);
    maturity = x(1,2:end);
    coupons = x(2:end,2:end);
    
    plotyr = 1990;
    plotyrindx = find(year==plotyr);
    
    figure
    hold on
    box on
    matstr = cumsum(coupons,2);
    fracstr = matstr./matstr(:,end)*100;
    plot(year,fracstr(:,[1]),'color',[0 0 0 ]+0.7,'linewidth',2)
    plot(year,fracstr(:,[3]),'color',[0 0 0 ]+0.5,'linewidth',2)
    plot(year,fracstr(:,[5]),'color',[0 0 0 ]+0.3,'linewidth',2)
    plot(year,fracstr(:,[10]),'color','k','linewidth',2)
    plot(year,fracstr(:,[end]),'-k','linewidth',2)
    text(year(plotyrindx),fracstr(plotyrindx,1)-12,'0-1','color',[0 0 0 ]+0.7,'fontsize',font_size);
    text(year(plotyrindx),fracstr(plotyrindx,3)-10,'1-3','color',[0 0 0 ]+0.5,'fontsize',font_size);
    text(year(plotyrindx),fracstr(plotyrindx,5)-4,'3-5','color',[0 0 0 ]+0.3,'fontsize',font_size);
    text(year(plotyrindx),fracstr(plotyrindx,10)-5,'5-10','color',[0 0 0 ],'fontsize',font_size);
    text(year(plotyrindx),fracstr(plotyrindx,end)-10,'10-40','color',[0 0 0 ],'fontsize',font_size);
    
    axis([1900 2022 0 105])
    %xlabel('Date');
    ylabel('Percent')
    
    print -depsc2 mat_str.eps
    
end;

    
function yt=simple_model(alph,rhos,rho,b_s,H);
A = [ 0 alph rhos ;
    0 rho^(-1)*(1-alph) -rho^(-1)*rhos ;
    0   0                 rhos];
B = -[ 1;
    rho^(-1)*(b_s-1);
    1];

yt = zeros(H,3);
yt(2,:) = B';
for t = 3:H;
    yt(t,:) = A*yt(t-1,:)';
end;
end






