% solve_ctime_model_new
% scrpt to solve continuous time linear model
% includes q and partial adjustment interest rate rule 
% Makes
% Figure 5.11 p. 175
% Figure 5.12 p. 178
% Figure 5.13 p. 179

clear variables
close all

font_size=16; 
gray1 = [0 0 0]+0.5;
bw_plots=1; 

dt = 0.01; 
T = 10; 

% set parameters
k = 2;
sig = 0.5;
r = 0.05;
rho = 0.05;
zetai = 1; 
phii = 0.3 ; % 0.4; Warning: do not use 0. Use 0.0001 instead. 
phis = 0.6;
om = 99; % 99 or more is code for infinity
thsx = 1;
thspi = 1; 
thix = 0.0;
thipi = 0.0;
thie = 0.25;
%thsx = 0; thspi = 0; thix = 0; thipi = 0; 
alph = 0; % use zero for surplus that does not respond to debt 
bs = 0.15;
bi = 0.6;
if alph == 0; 
    bs=0;
    bi=0; 
end;
if (alph > 0) & (alph < r);
    disp('program not set up for alpha between 0 and r.')
end;

% monetary shock

epsilon = [ 1; 0]; % money, then fiscal$

[xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);

% Figure 5.11 p. 175

figure;
%plot(t,[[rt at rhot pit]]/max(em,etau),'linewidth',2);
t = (0:dt:T)';
hold on
if bw_plots;
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-','color',gray1,'linewidth',2);
    text(4,0.15,'x','color',gray1,'fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'--k','linewidth',2);
    text(0,0.4,'i','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-k','linewidth',2);
    text(1.5,0.375,'\pi','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-k','linewidth',2);
    text(0.5,-0.8,'s, \alpha=0','color','k','fontsize',font_size);
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:0.5/dt:end),[xt(1:0.5/dt:end)],'s','color',gray1,'linewidth',2);
    %plot(t(1:0.5/dt:end),[it(1:0.5/dt:end)],'v','color','k','linewidth',2);
    %plot(t(1:0.5/dt:end),[pit(1:0.5/dt:end)],'o','color','k','linewidth',2);
    plot(t(1:0.5/dt:end),[st(1:0.5/dt:end)],'^','color','k','linewidth',2);
    
    plot(t,0*t,'-k');
    %legend('x','i','\pi','s')
else;
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',2);
    text(4,0.15,'x','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',2);
    text(0,0.4,'i','color','b','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',2);
    text(1.5,0.375,'\pi','color','r','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-m','linewidth',2);
    text(0.5,-0.8,'s, \alpha=0','color','m','fontsize',font_size);
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[xt(1:1/dt:end)],'s','color','k','linewidth',2);
    plot(t(1:1/dt:end),[it(1:1/dt:end)],'v','color','b','linewidth',2);
    plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','r','linewidth',2);
    plot(t(1:1/dt:end),[st(1:1/dt:end)],'^','color','m','linewidth',2);
    
    plot(t,0*t,'-k');
end;
xlabel('Time')
ylabel('Percent');
    
    disp('Monetary shock decomposition piterm = sterm + drterm, and sum of sterm+dterm');
    disp([piterm sterm drterm sterm+drterm]);
    
    % Same shock, add fiscal response
    
    alph = 0.4;
    bi = -pit(1);
    disp('using bi=');
    disp(bi);
    [xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
        findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);
    
    %figure;
    hold on
    box on
    if bw_plots;
        % first three show same result 
       % plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',1);
       % plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',1);
       % plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',1);
        plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'--','color',gray1,...
            'linewidth',2);
        text(3.5,0.65,'s, \alpha=0.4','color',gray1,'fontsize',font_size);
        plot(t(1:0.5/dt:end),[st(1:0.5/dt:end)],'^','color',gray1,'linewidth',2);

    else;
        plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',1);
        plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',1);
        plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',1);
        plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'--m','linewidth',2);
        text(3.5,0.65,'s, \alpha=0.4','color','m','fontsize',font_size);
    end;

axis([-0.5 6 -1.5 0.8]);
print -depsc2 surplus_theta_rule_1.eps


disp('Monetary shock decomposition piterm = sterm + drterm, and sum of sterm+dterm'); 
disp([piterm sterm drterm sterm+drterm]);

% Same shock and alpha, beta_i=0 

bi = 0; 
[xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);

%%%%%%%%%%%%

% Figure 5.12 p. 178

figure;
box on;
hold on
if bw_plots;
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-','color',gray1','linewidth',2);
    text(1,-0.15,'x','color',gray1,'fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'--k','linewidth',2);
    text(0.5,0.6,'i','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-k','linewidth',2);
    text(2.2,0.525,'\pi','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-k','linewidth',2);
    text(4.1,0.2,'s, \alpha=0.4, \beta_i=0','color','k','fontsize',font_size);
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:0.5/dt:end),[xt(1:0.5/dt:end)],'s','color',gray1,'linewidth',2);
   % plot(t(1:0.5/dt:end),[it(1:0.5/dt:end)],'v','color','k','linewidth',2);
   % plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','k','linewidth',2);
    plot(t(1:0.5/dt:end),[st(1:0.5/dt:end)],'^','color','k','linewidth',2);
else;
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',2);
    text(1,-0.15,'x','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',2);
    text(0.5,0.6,'i','color','b','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',2);
    text(2,0.525,'\pi','color','r','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-m','linewidth',2);
    text(4.1,0.2,'s, \alpha=0.4, \beta_i=0','color','m','fontsize',font_size);
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[xt(1:1/dt:end)],'s','color','k','linewidth',2);
    plot(t(1:1/dt:end),[it(1:1/dt:end)],'v','color','b','linewidth',2);
    plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','r','linewidth',2);
    %plot(t(1:1/dt:end),[st(1:1/dt:end)],'^','color','m','linewidth',2);
end;
plot(t,0*t,'-k');
ylabel('Percent');
%legend('x','i','\pi','s')
xlabel('Time')
axis([-0.5 6 -0.3 0.7]); 
print -depsc2 surplus_theta_rule_2.eps

disp('Monetary shock decomposition piterm = sterm + drterm, and sum of sterm+dterm'); 
disp([piterm sterm drterm sterm+drterm]);


% Same shock and alpha, find beta_i for no surplus impact 

bi = -0.5085; % manual search to find no cahnge in sum of surpluses 
disp('bi');
disp(bi)
[xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);

% Figure 5.13 p. 179
figure;
hold on
box on; 
if bw_plots
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-','color',gray1,'linewidth',2);
    text(1,-0.05,'x','color',gray1,'fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'--k','linewidth',2);
    text(0.65,0.62,'i','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-k','linewidth',2);
    text(2,0.52,'\pi','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-k','linewidth',2);
    text(6,-0.1,'s, \alpha=0.4, \beta_i=-0.51','color','k',...
        'fontsize',font_size,'backgroundcolor','w');
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:0.5/dt:end),[xt(1:0.5/dt:end)],'s','color',gray1,'linewidth',2);
  %  plot(t(1:0.5/dt:end),[it(1:0.5/dt:end)],'v','color','k','linewidth',2);
    %plot(t(1:0.5/dt:end),[pit(1:0.5/dt:end)],'o','color','k','linewidth',2);
    plot(t(1:1/dt:end),[st(1:1/dt:end)],'^','color','k','linewidth',2);
else;
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',2);
    text(1,-0.05,'x','color','k','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',2);
    text(0.6,0.6,'i','color','b','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',2);
    text(2,0.5,'\pi','color','r','fontsize',font_size);
    plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-m','linewidth',2);
    text(3,0.2,'s, \alpha=0.4, \beta_i=-0.51','color','m','fontsize',...
        font_size,'backgroundcolor','w');
    
    %colorvec = get(gcf,'defaultaxescolororder');
    plot(t(1:1/dt:end),[xt(1:1/dt:end)],'s','color','k','linewidth',2);
    plot(t(1:1/dt:end),[it(1:1/dt:end)],'v','color','b','linewidth',2);
    plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','r','linewidth',2);
    %plot(t(1:1/dt:end),[st(1:1/dt:end)],'^','color','m','linewidth',2);
end;
plot(t,0*t,'-k');
ylabel('Percent');
%legend('x','i','\pi','s')
xlabel('Time')
axis([-0.5 10 -0.2 0.7]); 
print -depsc2 surplus_theta_rule_3.eps


disp('Monetary shock decomposition piterm = sterm + drterm, and sum of sterm+dterm'); 
disp([piterm sterm drterm sterm+drterm]);

stop
% Full model monetary and fiscal shocks

k = 0.25;
sig = 1;
r = 0.01;
rho = 0.01;
zetai = 1; 
phii = 0.3 ; % 0.4; Warning: do not use 0. Use 0.0001 instead. 
phis = 0.5;
om = 0.2; % 99 or more is code for infinity
thsx = 1;
thspi = 0.25;
thix = 0.5;
thipi = 0.8;
thie = 0.0; 
alph = 0.2; % use zero for surplus that does not respond to debt 
bs = 0.15;
bi = 0.69;
if alph == 0; 
    bs=0;
    bi=0; 
end;
if (alph > 0) & (alph < r); 
    disp('program not set up for alpha between 0 and r.')
end;

% monetary shock 

epsilon = [ 1; 0]; % money, then fiscal$ 

[xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);

figure;
%plot(t,[[rt at rhot pit]]/max(em,etau),'linewidth',2);
t = (0:dt:T)';
hold on
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',2);
%text(4,-0.2,'x','color','k','fontsize',font_size);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'-b','linewidth',2);
%text(0.5,1.2,'i','color','b','fontsize',font_size);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',2);
%text(1.5,0.5,'\pi','color','r','fontsize',font_size);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'-m','linewidth',2);
%text(0.5,-2,'s, \alpha=0','color','m','fontsize',font_size);

colorvec = get(gcf,'defaultaxescolororder');
plot(t(1:1/dt:end),[xt(1:1/dt:end)],'s','color','k','linewidth',2);
plot(t(1:1/dt:end),[it(1:1/dt:end)],'v','color','b','linewidth',2);
plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','r','linewidth',2);

plot(t,0*t,'-k');
ylabel('Percent');
legend('x','i','\pi','s')
xlabel('Time')

disp('Monetary shock decomposition piterm = sterm + drterm, and sum of sterm+dterm'); 
disp([piterm sterm drterm sterm+drterm]);

% fiscal shock 

epsilon = [ 0; -1]; % money, then fiscal$ 

[xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T);

figure;
%plot(t,[[rt at rhot pit]]/max(em,etau),'linewidth',2);
t = (0:dt:T)';
hold on
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ xt]],'-k','linewidth',2);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[ it]],'--b','linewidth',2);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[pit ]],'-r','linewidth',2);
plot([(-1:dt:0)'; t],[ (-1:dt:0)'*[ 0];[st ]],'--m','linewidth',2);

colorvec = get(gcf,'defaultaxescolororder');
plot(t(1:1/dt:end),[xt(1:1/dt:end)],'s','color','k','linewidth',2);
plot(t(1:1/dt:end),[it(1:1/dt:end)],'v','color','b','linewidth',2);
plot(t(1:1/dt:end),[pit(1:1/dt:end)],'o','color','r','linewidth',2);
plot(t(1:1/dt:end),[st(1:1/dt:end)],'^','color','m','linewidth',2);

plot(t,0*t,'-k');
ylabel('Percent');
legend('x','i','\pi','s')
xlabel('Time')

disp('Fiscal shock decomposition piterm = sterm + drterm, and sum of sterm+dterm'); 
disp([piterm sterm drterm sterm+drterm]);

%%%%%%%%%%%
% function definitions
%%%%%%%%%%

function [xt, pit, qt, vt, it, uit, ust, st, piterm, sterm, drterm ] =...
    findir(k, sig, r, rho, zetai, phii, phis, om, thsx, thspi, thix, thipi, thie, alph, epsilon, bs, bi, dt,T)
% set up matrix
if (alph > 0) & (alph < r);
    disp('case that alpha > 0 and < r not implemented')
end;
if alph == 0; 
    if (~(bi==0)) | (~(bs ==0));
        disp('alpha = 0. setting bi and bs to 0. you should do this')
        bi=0;
        bs=0;
    end
end
if om < 99; 
%           x           pi       q          v        i        ui    us
A = [       0          -sig      0          0       sig        0       0 ; % x
           -k           rho      0          0         0        0       0 ; %pi
            0             0   r+om          0         1        0       0 ; %q
        -thsx      -1-thspi      0     r-alph         1        0      -1 ; %v
    zetai*thix  zetai*thipi      0          0    -zetai    zetai       0 ; %i
            0             0      0          0         0    -phii       0 ; %ui 
            0             0      0          0         0       0    -phis ];% us

B = [  0   0 ;
     -bi -bs ;
       0   0 ;
       0   0 ;
    thie   0 ;
       1   0 ;
       0   1 ];
   
   if alph > r;
       C = [1 0 ;
            0 0 ;
            0 1 ;
            0 1 ;
            0 0 ;
            0 0 ;
            0 0 ];
   elseif alph ==0;
       C = [1 0 0 ;
            0 1 0 ;
            0 0 1 ;
            0 0 1 ;
            0 0 0 ;
            0 0 0 ;
            0 0 0 ];
   end;
end

if om>=99; % code for infinite om, i.e. short term debt. Drop Q equation 
    %       x          pi               v        i     ui     us
A = [       0        -sig               0       sig     0     0 ; % x
           -k         rho               0         0     0     0 ; %pi
        -thsx    -1-thspi          r-alph         1     0    -1 ; %v
   zetai*thix zetai*thipi               0    -zetai zetai     0 ; %i
            0           0               0         0 -phii     0 ;% ui
            0           0               0         0     0 -phis ];% us
B = [  0   0 ;
     -bi -bs ;
       0   0 ;
       thie   0 ;
       1   0 ;
       0   1 ];
   
   if alph > r;
       C = [1  ;
            0  ;
            0  ;
            0  ;
            0  ;
            0  ];
   elseif alph ==0;
       C = [1 0  ;
            0 1  ;
            0 0  ;
            0 0  ;
            0 0  ;
            0 0 ];
   end;
end


N = size(A,1); 
Ne = size(B,2); 
Nd = size(C,2); 
[Q, Lam] = eig(A);
eigs = diag(Lam);
posindx = find(real(diag(Lam))>0); %index of positive eigenvalues
nonposindx = find(real(diag(Lam))<=0);
invQ = inv(Q); 

if 0;
    disp('Real | imaginary part of Eigenvalues');
    disp(real(diag(Lam)'));
    disp(imag(diag(Lam)'));
    disp('number of unstable eigenvalues');
    disp(size(posindx,1));
    disp('number of expectational errors');
    disp(Nd);
    if ~(Nd == size(posindx,1)); 
        disp('error, number of deltas not equal to number of unstable eigenvalues');
    end
    disp('A');
    disp(A);
    disp('Q');
    disp(Q);
end

% selecting rows method
if 0;
    invQ_plus = invQ(posindx,:);
    invQ_minus = invQ(nonposindx,:);
    invQ_plus*C
    bigmat = invQ_minus*(eye(N)-C*inv(invQ_plus*C)*invQ_plus)*B;
    
    Lam_plus = Lam(posindx,posindx);
    Lam_minus = Lam(nonposindx,nonposindx);
    lam_plus = diag(Lam_plus);
    lam_minus = diag(Lam_minus);
    
    t = (0:dt:T)';
    Nt = size(t,1);
    z_minus_t = zeros(Nt,N-Nd);
    for tindx = 1:size(t,1);
        z_minus_t(tindx,:) = (diag(exp(lam_minus*t(tindx)))*bigmat*epsilon)';
    end;
    zt = zeros(Nt,N);
    zt(:,nonposindx) = z_minus_t;
    
    Xt = zt*Q';
end;

% P R version -- gives same result
if 1;
    P = eye(N);
    M = eye(N);
    P = P(posindx,:);
    M = M(nonposindx,:);
    
    bigmat2 = M'*M*invQ*(eye(N)-C*inv(P*invQ*C)*P*invQ)*B;
    biglam = diag(M'*M*Lam*M'*M); % Lambdastar diagonals placed in a vector 
   
    t = (0:dt:T)';
    Nt = size(t,1);
    zt2 = zeros(Nt,N);
    for tindx = 1:size(t,1);
        zt2(tindx,:) = (diag(exp(biglam*t(tindx)))*bigmat2*epsilon)';
    end;
    Xt2 = zt2*Q';
    Xt = Xt2;
    zt = zt2; 
end;

% extract series 


xt = Xt(:,1);
pit = Xt(:,2);
if om < 99;
    qt = Xt(:,3) ;
    vt = Xt(:,4);
    it = Xt(:,5);
    uit = Xt(:,6);
    ust = Xt(:,7);
else;
    vt = Xt(:,3);
    it = Xt(:,4);
    uit = Xt(:,5);
    ust = Xt(:,6);
    qt = 0*it;
end;

st = thspi*pit + thsx*xt + alph*vt + ust;

% terms of identity. These are a check, not as numerically accurate
if 0;
    piterm2 = sum(exp(-(r+om)*(t+dt)).*pit*dt);
    sterm2 = -sum(exp(-r*(t+dt)).*st*dt);
    drterm2 = sum(exp(-r*(t+dt)).*(1-exp(-om*(t+dt))).*(it-pit)*dt); % leave r out as deviations from ss
    disp('Direct sum piterm sterm drterm sterm+drterm') 
    disp([piterm2 sterm2 drterm2 sterm2+drterm2]);


end;

% terms of identity computed from solution 

if om < 99;
    ax  = [ 1 0 0 0 0 0 0]';
    api = [ 0 1 0 0 0 0 0]';
    av  = [ 0 0 1 0 0 0 0]';
    aq  = [ 0 0 0 1 0 0 0]';
    ai  = [ 0 0 0 0 1 0 0]';
    aui = [ 0 0 0 0 0 1 0]';
    aus = [ 0 0 0 0 0 0 1]';
    as = thspi*api + thsx*ax + alph*av + aus;
    
    lamr = 0*biglam;
    lamrw = 0*biglam;
    
    for indx = 1:size(biglam,1);
        if biglam(indx) < 0;
            lamr(indx) = 1/(r-biglam(indx));
            lamrw(indx) = 1/(r+om-biglam(indx));
        end;
    end;
    
    piterm = api'*Q*diag(lamrw)*bigmat2*epsilon;
    sterm = -as'*Q*diag(lamr)*bigmat2*epsilon;
    drterm = (ai-api)'*Q*(diag(lamr)-diag(lamrw))*bigmat2*epsilon;
    
else; % omega = infinity case coded as 99
    ax  = [ 1 0 0 0 0 0]';
    api = [ 0 1 0 0 0 0]';
    av  = [ 0 0 1 0 0 0]';
    ai =  [ 0 0 0 1 0 0]';
    aui = [ 0 0 0 0 1 0]';
    aus = [ 0 0 0 0 0 1]';
    as = thspi*api + thsx*ax + alph*av + aus;
end;

lamr = 0*biglam;
lamrw = 0*biglam;

for indx = 1:size(biglam,1);
    if biglam(indx) < 0;
        lamr(indx) = 1/(r-biglam(indx));
        lamrw(indx) = 1/(r+om-biglam(indx));
    end;
end;

piterm = 0;
sterm = -as'*Q*diag(lamr)*bigmat2*epsilon;
drterm = (ai-api)'*Q*diag(lamr)*bigmat2*epsilon;


end
